<?php

use Shopware\Components\CacheManager;

/**
 * Class Shopware_Plugins_Backend_OfficialEkomiIntegration_Bootstrap.
 */
class Shopware_Plugins_Backend_OfficialEkomiIntegration_Bootstrap extends Shopware_Components_Plugin_Bootstrap
{
    /**
     * The Url to validate shop.
     */
    const URL_GET_SETTINGS = 'https://api.ekomi.de/v3/getSettings';

    /**
     * Ekomi Terms and Conditions Url
     */
    const EKOMT_TERMS_AND_CONDITIONS_URL = 'https://plugins-dashboard.ekomiapps.de/terms_and_conditions';

    /**
     * Booking url.
     */
    const EKOMI_BOOKING_URL = 'http://ssi.ekomi.com/booking';

    /**
     * The URL where the order data is sent.
     */
    const URL_TO_SEND_DATA = 'https://plugins-dashboard.ekomiapps.de/api/v1/order';

    /**
     * Plugin status values.
     */
    const PLUGIN_STATUS_ENABLE = 'yes';
    const PLUGIN_STATUS_DISABLE = 'no';
    const PLUGIN_TERMS_AND_CONDITIONS_AGREED = 'yes';

    /**
     * Product Identifiers.
     */
    const PRODUCT_IDENTIFIER_ID = 'id';
    const PRODUCT_IDENTIFIER_SKU = 'sku';
    const PRODUCT_IDENTIFIER_EAN = 'ean';

    /**
     * Maximum turnaround time.
     */
    const MAX_TURNAROUND_TIME = 30;

    /**
     * Cron time interval.
     */
    const CRON_TIME_INTERVAL = 3600;

    /**
     * Maximum shopware version.
     */
    const MAX_SHOPWARE_VERSION = 5;

    /**
     * Plugin name in PD.
     */
    const PLUGIN_NAME = 'shopware';

    /**
     * Plugin current version
     */
    const PLUGIN_VERSION = '2.9.4';

    /**
     * Exports methods.
     */
    const EXPORT_METHOD_HOOK = 'hook';
    const EXPORT_METHOD_CRON = 'cron';

    /**
     * Get settings api response.
     */
    const GET_SETTINGS_RESPONSE = 'Access denied';

    /**
     * Order state action.
     */
    const ACTION_BATCH_PROCESS = 'batchProcess';
    const ACTION_BATCH_SAVE = 'save';

    /**
     * Log exception.
     */
    const LOG_EXCEPTION = 'Ekomi Plugin Exception! ';

    /**
     * Maximim limit of fetching orders
     */
    const MAX_RESULT_LIMIT = 50;

    /**
     * Orders histories.
     *
     * @var array
     */
    private $ordersHistory = array();
    /**
     * Gets the version of plugin.
     *
     * @return string
     */
    public function getVersion()
    {
        return self::PLUGIN_VERSION;
    }

    /**
     * Gets the Label of plugin.
     *
     * @return string
     */
    public function getLabel()
    {
        return 'eKomi Integration';
    }

    /**
     * Gets plugin description.
     *
     * @return string
     */
    public function getDescription()
    {
        $description = 'eKomi Plugin for Shopware allows you to integrate your Shopware shop easily with eKomi system. ';
        $description .= 'This allows you to collect verified reviews, display eKomi seal on your website and get your ';
        $description .= 'seller ratings on Google. This helps you increase your website\'s click through rates, ';
        $description .= 'conversion rates and also, if you are running Google AdWord Campaigns, this helps in improving your ';
        $description .= 'Quality Score and hence your costs per click.';

        return $description;
    }

    /**
     * Gets the Plugin information.
     *
     * @return array
     */
    public function getInfo()
    {
        return array(
            'version' => $this->getVersion(),
            'label' => $this->getLabel(),
            'author' => 'eKomi Ltd',
            'copyright' => 'Copyright (c) eKomi Ltd',
            'license' => '(c) eKomi Ltd, 2018',
            'supplier' => 'eKomi',
            'description' => $this->getDescription(),
            'support' => 'eKomi Support',
            'link' => self::EKOMI_BOOKING_URL,
        );
    }

    /**
     * Creates the configuration form for the plugin.
     */
    private function createConfig()
    {
        $statusValues = $this->gePluginStatusValues();
        $this->Form()->setElement(
            'select',
            'plugin-enabled',
            array(
                'label' => 'Plugin Enabled',
                'store' => $statusValues,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => $statusValues[0][0],
                'multiSelect' => false,
            )
        );
        $this->Form()->setElement(
            'checkbox',
            'terms-and-conditions',
            array(
                'label' => 'Accept Terms and Conditions listed <a href="' . self::EKOMT_TERMS_AND_CONDITIONS_URL . '" target="_blank">here</a>',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => true,
                'description' => 'Terms and Conditions of Ekomi. If they are not accepted the plugin will not send data to Ekomi',
            )
        );
        $this->Form()->setElement(
            'text',
            'shop-id',
            array(
                'label' => 'Shop ID',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
            )
        );
        $this->Form()->setElement(
            'text',
            'shop-secret',
            array(
                'label' => 'Shop Secret',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
            )
        );
        $statusList = $this->getOrderStatuses();
        $this->Form()->setElement(
            'select',
            'order-status',
            array(
                'label' => 'Order Status',
                'store' => $statusList,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => $statusList[2][0],
                'multiSelect' => true,
            )
        );
        $this->Form()->setElement(
            'boolean',
            'product-reviews',
            array(
                'value' => true,
                'label' => 'Product Reviews',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
            )
        );
        $modeList = $this->geModeList();
        $this->Form()->setElement(
            'select',
            'api-mode',
            array(
                'label' => 'Collection Method',
                'store' => $modeList,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => $modeList[1][0],
                'multiSelect' => false,
            )
        );
        $identifierList = $this->geProductIdentifierList();
        $this->Form()->setElement(
            'select',
            'product-identifier',
            array(
                'label' => 'Product Identifier',
                'store' => $identifierList,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => $identifierList[0][0],
                'multiSelect' => false,
            )
        );
        $this->Form()->setElement(
            'text',
            'exclude-products',
            array(
                'label' => 'Exclude Products',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
            )
        );
        $daysList = $this->getTurnAroundTimeOptions();
        $this->Form()->setElement(
            'select',
            'turnaround-time',
            array(
                'label' => 'Fetch Orders for Last "x" Days',
                'store' => $daysList,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => $daysList[0][0],
                'multiSelect' => false,
            )
        );
        $this->Form()->setElement(
            'text',
            'order-last-updated',
            array(
                'label' => 'Order Last Updated',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
            )
        );
        $exportMethod = array(
            array(self::EXPORT_METHOD_HOOK, 'On Order Status Change'),
            array(self::EXPORT_METHOD_CRON, 'Every Hour (Cron)'),
        );
        $this->Form()->setElement(
            'select',
            'export-method',
            array(
                'label' => 'Export Method',
                'store' => $exportMethod,
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => $exportMethod[1][0],
                'multiSelect' => false,
            )
        );
        $this->Form()->setElement(
            'checkbox',
            'show-prc-widget',
            array(
                'label' => 'Show PRC Widget',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'value' => false,
                'description' => 'The eKomi Smart PRC widget will be shown in product detail page.',
                'disabled' => true
            )
        );
        $this->Form()->setElement(
            'text',
            'prc-widget-token',
            array(
                'label' => 'PRC Widget Token',
                'scope' => \Shopware\Models\Config\Element::SCOPE_SHOP,
                'disabled' => true
            )
        );

        $this->addFormTranslations($this->getConfigurationTranslations());
    }

    /**
     * Gets translation for configuration form.
     *
     * @return array
     */
    private function getConfigurationTranslations()
    {
        $translation = array(
            'de_DE' => array(
                'plugin-enabled' => array(
                    'label' => 'Plug-in aktiviert',
                ),
                'shop-id' => array(
                    'label' => 'Shop ID',
                ),
                'shop-secret' => array(
                    'label' => 'Shop Passwort',
                ),
                'order-status' => array(
                    'label' => 'Bestellstatus',
                ),
                'product-reviews' => array(
                    'label' => 'Produktbewertungen',
                ),
                'api-mode' => array(
                    'label' => 'Sammelmethode',
                ),
                'product-identifier' => array(
                    'label' => 'Produktkennung',
                ),
                'exclude-products' => array(
                    'label' => 'Produkte ausschließen',
                ),
                'turnaround-time' => array(
                    'label' => 'Bestellungen für die letzten "x" Tage abrufen',
                ),
                'order-last-updated' => array (
                    'label' => 'Bestellung zuletzt aktualisiert',
                ),
                'export-method' => array(
                    'label' => 'Export Methode',
                ),
                'show-prc-widget' => array(
                    'label' => 'PCR Widget anzeigen',
                ),
                'prc-widget-token' => array(
                    'label' => 'PCR Widget Token',
                ),
                'terms-and-conditions' => array(
                    'laebl' => 'Akzeptieren Sie die <a href="' . self::EKOMT_TERMS_AND_CONDITIONS_URL .'" target="_blank">hier</a> aufgeführten allgemeinen Geschäftsbedingungen'
                ),
            ),
        );

        return $translation;
    }

    /**
     * Gets the modes list.
     *
     * @return array
     */
    public function geModeList()
    {
        $modeList = array(
            array('sms', 'SMS'),
            array('email', 'Email'),
            array('fallback', 'SMS if contact number, otherwise Email'),
        );

        return $modeList;
    }

    /**
     * Gets the product Identifier.
     *
     * @return array
     */
    public function geProductIdentifierList()
    {
        $list = array(
            array(self::PRODUCT_IDENTIFIER_ID, 'Id'),
            array(self::PRODUCT_IDENTIFIER_SKU, 'Sku'),
            array(self::PRODUCT_IDENTIFIER_EAN, 'Ean'),
        );

        return $list;
    }

    /**
     * Gets the product Identifier.
     *
     * @return array
     */
    public function gePluginStatusValues()
    {
        $statuses = array(
            array(self::PLUGIN_STATUS_DISABLE, 'No'),
            array(self::PLUGIN_STATUS_ENABLE, 'Yes'),
        );

        return $statuses;
    }

    /**
     * Gets all order status.
     *
     * @return array
     */
    public function getOrderStatuses()
    {
        $filters = array(array('property' => 'status.id', 'expression' => '!=', 'value' => '-1'));
        $repository = Shopware()->Models()->getRepository('Shopware\Models\Order\Order');
        $orderStatuses = $repository->getOrderStatusQuery($filters)->getArrayResult();
        $statusList = array();
        foreach ($orderStatuses as $status) {
            $name = $status['name'];
            if (is_null($name)) {
                $name = strip_tags($status['description']);
            }
            $statusList[] = array($status['id'], $name);
        }

        return $statusList;
    }

    /**
     * Gets turnaround list.
     *
     * @return array
     */
    public function getTurnAroundTimeOptions()
    {
        $daysList = array();
        for ($day = self::MAX_TURNAROUND_TIME; $day > 0; $day -= 10) {
            $daysList[] = array($day, $day);
        }

        return $daysList;
    }

    /**
     * Installs the plugin.
     *
     * @return array|bool
     *
     * @throws Exception
     */
    public function install()
    {
        $this->_deleteCronEntry();
        $this->createCronJob('Ekomi Order Export', 'EkomiExport', self::CRON_TIME_INTERVAL, true, false);

        $this->subscribeEvent(
            'Shopware_CronJob_EkomiExport',
            'onEkomiExport'
        );
        $this->subscribeEvent(
            'Shopware_Controllers_Backend_OrderState_Notify',
            'sendOrderToEkomi'
        );
        $this->subscribeEvent(
            'Enlight_Controller_Action_PostDispatchSecure_Frontend',
            'onFrontendPostDispatch'
        );

        $this->registerController('widgets', 'officialekomiintegration');
        $this->createConfig();

        return array('success' => true, 'invalidateCache' => array('backend'));
    }

    /**
     * Deletes the cron entry.
     */
    protected function _deleteCronEntry()
    {
        $connection = $this->get('dbal_connection');
        $connection->delete(
            's_crontab',
            array('action' => 'EkomiExport')
        );

        $connection->delete(
            's_crontab',
            array('action' => 'Shopware_CronJob_EkomiExport')
        );
    }

    /**
     * Updates the plugin.
     *
     * @param string $version
     *
     * @return array|bool
     *
     * @throws Exception
     */
    public function update($version)
    {
        $this->_deleteCronEntry();

        return $this->install();
    }

    /**
     * Assigns value in view.
     *
     * @param Enlight_Event_EventArgs $arguments
     */
    public function onFrontendPostDispatch(Enlight_Event_EventArgs $arguments)
    {
        $view = $arguments->getSubject()->View();
        $view->assign('isEkomiPluginInstalled', true);
    }

    /**
     * Enables the plugin.
     *
     * @return array|bool
     *
     * @throws Exception
     */
    public function enable()
    {
        $shops = $this->getSubShops();
        $success = false;
        foreach ($shops as $shop) {
            $configurationData = $this->getConfigurationData($this->Collection()->getConfig($this->getName(), $shop));
            if (self::PLUGIN_STATUS_ENABLE == $configurationData['plugin-enabled']) {
                if (!empty($configurationData['shop-id']) &&
                    !empty($configurationData['shop-secret']) &&
                    $this->verifyAccount($configurationData)
                ) {
                    $success = true;
                } else {
                    $this->setDefaultConfigurations($shop);
                }
            }
        }

        if (Shopware()->Container()->get('config')->get('version') < self::MAX_SHOPWARE_VERSION) {
            return $success;
        } else {
            if (!$success) {
                throw new \Exception('Invalid Shop ID & Secret!');
            }

            return array('success' => $success);
        }
    }

    /**
     * Sets default values in plugin configuration fields.
     *
     * @param array $shop
     *
     * @return bool
     */
    public function setDefaultConfigurations($shop)
    {
        $pluginManager = $this->_getPluginManager();
        $pluginManager->saveConfigElement($this, 'plugin-enabled', self::PLUGIN_STATUS_DISABLE, $shop);
        $pluginManager->saveConfigElement($this, 'shop-id', '', $shop);
        $pluginManager->saveConfigElement($this, 'shop-secret', '', $shop);

        return true;
    }

    /**
     * @return object|\Shopware\Bundle\PluginInstallerBundle\Service\InstallerService
     */
    protected function _getPluginManager()
    {
        return Shopware()->Container()->get('shopware_plugininstaller.plugin_manager');
    }

    /**
     * Sets order last updated date time in configuration value.
     *
     * @param array  $shop
     * @param string $datetime
     *
     * @return bool
     */
    public function setOrderLastUpdatedDatetime($shop, $datetime)
    {
        $pluginManager = $this->_getPluginManager();
        $pluginManager->saveConfigElement(
            $pluginManager->getPluginByName($this->getName()),
            'order-last-updated',
            $datetime,
            $shop
        );

        return true;
    }

    /**
     * Gets order data from database.
     *
     * @param int $orderId
     *
     * @return array
     */
    public function getOrderData($orderId)
    {
        $orderData = Shopware()->Models()
            ->getRepository('Shopware\Models\Order\Order')
            ->getOrdersQuery(
                array(
                    array(
                        'property' => 'orders.id',
                        'value' => $orderId,
                    ),
                ),
                array()
            )
            ->getOneOrNullResult(\Doctrine\ORM\AbstractQuery::HYDRATE_ARRAY);

        return $orderData;
    }

    /**
     * Sends order data to eKomi.
     *
     * @param Enlight_Event_EventArgs $arguments
     *
     * @throws Exception
     */
    public function sendOrderToEkomi(Enlight_Event_EventArgs $arguments)
    {
        try {
            $parameter = $arguments->getSubject()->Request()->getParams();
            $action = $parameter['action'];
            if (self::ACTION_BATCH_PROCESS == $action) {
                foreach ($parameter['orders'] as $key => $order) {
                    $this->initProcessOrderData($order['id']);
                }
            } elseif (self::ACTION_BATCH_SAVE == $action) {
                $this->initProcessOrderData($parameter['id']);
            }
        } catch (Exception $exception) {
            $this->log($exception->getMessage(), self::LOG_EXCEPTION);
        }
    }

    /**
     * Initiates order process to sent to eKomi.
     *
     * @param int $orderId
     *
     * @throws Exception
     */
    public function initProcessOrderData($orderId)
    {
        $orderData = $this->getOrderData($orderId);
        if (!empty($orderData)) {
            $configurationData = $this->getConfigurationData($this->Config());
            if(isset($orderData['languageSubShop']['id'])) {
                $configurationData = $this->getConfigFromShopId($orderData['languageSubShop']['id']);
            }

            if ($this->checkIfExportMethodIsHook($configurationData)) {
                $this->sendOrderData($orderData, $configurationData);
            }
        }
    }

    /**
     * Checks if export is hook.
     *
     * @param array $configurationData
     *
     * @return bool
     */
    public function checkIfExportMethodIsHook($configurationData)
    {
        if (!empty($configurationData) &&
            self::PLUGIN_STATUS_ENABLE == $configurationData['plugin-enabled'] &&
            self::EXPORT_METHOD_HOOK == $configurationData['export-method']
        ) {
            return true;
        }

        return false;
    }

    /**
     * Adds scheme in URL.
     *
     * @param string $url
     * @param object $shop
     *
     * @return string
     */
    protected function addSchemeInURL($url, $shop)
    {
        $scheme = 'http://';
        if ($shop->getSecure()) {
            $scheme = 'https://';
        }

        return parse_url($url, PHP_URL_SCHEME) === null ? $scheme . $url : $url;
    }

    /**
     * Fills canonical url and image url.
     *
     * @param array  $orderData
     * @param array  $shop
     * @param string $method
     *
     * @return mixed
     */
    public function fillCanonicalUrls($orderData, $shop = null, $method = 'hook')
    {
        if (isset($orderData['details'])) {
            foreach ($orderData['details'] as $key => $detail) {
                $canonicalUrl = '';
                $imageUrl = '';
                try {
                    if (self::EXPORT_METHOD_CRON == $method) {
                        $frontendUrl = $shop->getHosts().$shop->getBasePath().'/';
                        $frontendUrl = $this->addSchemeInURL($frontendUrl, $shop);
                        $imageUrl = $this->getUrlRedirectedTo($frontendUrl.$this->getImageUrl($detail['articleId']));
                        $canonicalUrl = $this->getUrlRedirectedTo($frontendUrl.$this->getRewriteUrl($detail['articleId'], $shop->getId()));
                    } elseif (self::EXPORT_METHOD_HOOK == $method) {
                        $article = Shopware()->Modules()->Articles()->sGetArticleById($detail['articleId']);
                        if ($article) {
                            $imageUrl = $article['image']['source'];
                            $canonicalUrl = $article['linkDetailsRewrited'];
                        }
                    }
                } catch (Exception $exception) {
                    $this->log($exception->getMessage(), self::LOG_EXCEPTION);
                }

                $orderData['details'][$key]['imageUrl'] = $imageUrl;
                $orderData['details'][$key]['canonicalUrl'] = $canonicalUrl;
            }
        }

        return $orderData;
    }

    /**
     * Executes every 24 hours if Cron is selected as export method.
     *
     * @param Enlight_Event_EventArgs $arguments
     *
     * @return bool
     *
     * @throws Exception
     */
    public function onEkomiExport(Enlight_Event_EventArgs $arguments)
    {
        try {
            $shops = $this->getSubShops();
            foreach ($shops as $shop) {
                $configurationData = $this->getConfigurationData($this->Collection()->getConfig($this->getName(), $shop));
                if (self::EXPORT_METHOD_CRON != $configurationData['export-method'] ||
                    self::PLUGIN_STATUS_DISABLE == $configurationData['plugin-enabled'] ||
                    !$this->verifyAccount($configurationData)) {
                    continue;
                }

                $updateDate = $this->sendOrdersOfShop($configurationData, $shop);
                if (!empty($updateDate)) {
                    $this->setOrderLastUpdatedDatetime($shop, $updateDate);
                }
            }
        } catch (\Throwable $e) {
            $this->log($e->getMessage(), self::LOG_EXCEPTION);
        }

        return true;
    }

    /**
     * @param array $configurationData
     * @param \Shopware\Models\Shop\Shop $shop
     * @return string
     * @throws Exception
     */
    public function sendOrdersOfShop($configurationData, $shop)
    {
        $orders = $this->getOrdersForShop(
            $configurationData['order-status'],
            $this->getTurnAroundTime($configurationData),
            $this->getSubShopId($shop),
            $shop
        );

        $dateToSet = null;
        foreach ($orders as $order) {
            unset($order['customer']['hashPassword']);
            $this->sendOrderData($order, $configurationData, $shop, $configurationData['export-method']);
            if ($this->ordersHistory[$order['id']]['change_date'] > $dateToSet) {
                $dateToSet = $this->ordersHistory[$order['id']]['change_date'];
            }
        }

        if (!is_null($dateToSet)) {
            return date('Y-m-d H:i:s.u', strtotime($dateToSet));
        }

        return date('Y-m-d H:i:s.u', strtotime('now'));
    }

    /**
     * @param $configurationData
     * @return string
     * @throws Exception
     */
    public function getTurnAroundTime($configurationData)
    {
        if (!empty($configurationData['order-last-updated'])) {
            try {
                $turnAroundTime = new DateTime($configurationData['order-last-updated']);
                $turnAroundTime = $turnAroundTime->format('Y-m-d H:i:s.u');
            } catch (\Throwable $e) {
                $this->log($e->getMessage(), self::LOG_EXCEPTION);
                $turnAroundTime = new DateTime('-' . $configurationData['turnaround-time'] . ' day');
                $turnAroundTime = $turnAroundTime->format('Y-m-d') . ' 00:00:00';
            }
        } else {
            $turnAroundTime = new DateTime('-' . $configurationData['turnaround-time'] . ' day');
            $turnAroundTime = $turnAroundTime->format('Y-m-d') . ' 00:00:00';
        }

        return $turnAroundTime;
    }

    /**
     * Gets sub shop id.
     *
     * @param \Shopware\Models\Shop\Shop $shop
     *
     * @return int
     */
    public function getSubShopId($shop)
    {
        if (!is_null($shop->getMain())) {
            return $shop->getMain()->getId();
        }

        return $shop->getId();
    }

    /**
     * Gets image url.
     *
     * @param int $articleId
     *
     * @return string
     */
    public function getImageUrl($articleId)
    {
        $repository = Shopware()->Models()->getRepository('Shopware\Models\Article\Article');
        $article = $repository->find($articleId);
        if (is_object($article)) {
            $image = $article->getImages()->first();
            if (is_object($image)) {
                $media = $image->getMedia();
                if (is_object($media)) {
                    return $media->getPath();
                }
            }
        }

        return '';
    }

    /**
     * Gets reWrite url.
     *
     * @param int $articleId
     * @param int $shopId
     *
     * @return string
     */
    public function getRewriteUrl($articleId, $shopId)
    {
        $queryBuilder = Shopware()->Container()->get('dbal_connection')->createQueryBuilder();
        $queryBuilder->select('urls.path')
            ->from('s_core_rewrite_urls', 'urls')
            ->where('urls.org_path = :org_path')
            ->andWhere('urls.subshopID = :subshopID')
            ->setParameter(':org_path', "sViewport=detail&sArticle={$articleId}")
            ->setParameter(':subshopID', $shopId);

        try {
            $path = $queryBuilder->execute()->fetch(PDO::FETCH_COLUMN);
        } catch (\Exception $exception) {
            $path = '';
            $this->log($exception->getMessage(), 'Exception');
        }

        return ($path) ? $path : '';
    }

    /**
     * Gets sub shops.
     *
     * @return array
     */
    public function getSubShops()
    {
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select(array('shops'))
            ->from('Shopware\Models\Shop\Shop', 'shops');
        $builder->where('shops.active = 1');
        $query = $builder->getQuery();

        $shops = array();
        try {
            $shops = $query->getResult();
        } catch (\Exception $e) {
            $this->log($e->getMessage(), 'Exception');
        }

        return $shops;
    }

    /**
     * Gets configurations of the shop.
     *
     * @param int $id
     *
     * @return array
     */
    public function getConfigFromShopId($id)
    {
        $shop = Shopware()->Models()->getRepository('Shopware\Models\Shop\Shop')->find($id);
        if ($shop) {
            return $this->getConfigurationData($this->Collection()->getConfig($this->getName(), $shop));
        }

        return null;
    }

    /**
     * Gets order data for the shop which should be send to eKomi.
     *
     * @param array                      $orderStatuses
     * @param int                        $turnAroundTime
     * @param int                        $subShopId
     * @param \Shopware\Models\Shop\Shop $languageSubShop
     *
     * @return array
     *
     * @throws Exception
     */
    public function getOrdersForShop($orderStatuses, $turnAroundTime, $subShopId, $languageSubShop)
    {
        $dateRange = $this->getDateRange($turnAroundTime);
        $builder = Shopware()->Models()->getRepository('Shopware\Models\Order\Order')->getOrdersQueryBuilder();
        $builder->where(
                $builder->expr()->eq('orders.shopId', ':subShopId')
            )->andWhere(
                $builder->expr()->eq('orders.languageSubShop', ':languageSubShop')
            )->andWhere(
                $builder->expr()->in('orders.status', ':orderStatuses')
            )->andWhere(
                $builder->expr()->in('orders.orderTime BETWEEN :fromLimit AND :toLimit OR orders.id', ':orderIds')
            )->setParameter(
                'orderIds',
                $this->getOrderIdsFromHistory($orderStatuses, $dateRange)
            )->setParameter(
                'subShopId',
                $subShopId
            )->setParameter(
                'languageSubShop',
                $languageSubShop
            )->setParameter(
                'fromLimit',
                date_format($dateRange['from'],"Y-m-d H:i:s")
            )->setParameter(
                'toLimit',
                date_format($dateRange['to'],"Y-m-d H:i:s")
            )->setParameter(
                'orderStatuses',
                $orderStatuses
            );

        $ordersData = $builder->getQuery()->getArrayResult();
        return $ordersData;
    }

    /**
     * Gets order Ids.
     *
     * @param array $orderStatuses
     * @param array $dateRange
     *
     * @return array
     */
    protected function getOrderIdsFromHistory($orderStatuses, $dateRange){
        $subSelect = Shopware()->Container()->get('dbal_connection')->createQueryBuilder();
        $builder = Shopware()->Container()->get('dbal_connection')->createQueryBuilder()
            ->select(array('history.orderID', 'max(history.change_date) as change_date'))
            ->from('s_order_history', 'history')
            ->where(
                $subSelect->expr()->in('history.order_status_id', $orderStatuses)
            )->andWhere(
                $subSelect->expr()->gte('history.change_date', ':fromLimit')
            )->andWhere(
                $subSelect->expr()->lte('history.change_date', ':toLimit')
            )->groupBy(
                'history.orderID'
            )->setParameter(
                'fromLimit',
                date_format($dateRange['from'],"Y-m-d H:i:s")
            )->setParameter(
                'toLimit',
                date_format($dateRange['to'],"Y-m-d H:i:s")
            )->orderBy(
                'history.change_date','ASC'
            )->setMaxResults(
                self::MAX_RESULT_LIMIT
            );

        try {
            $histories = $builder->execute()->fetchAll();
            $orderIds = $this->parseOrderHistory($histories);
            return $orderIds;
        } catch (\Exception $exception) {
            $this->log($exception->getMessage(), self::LOG_EXCEPTION);
        }

        return array();
    }

    /**
     * Parses the order histories.
     *
     * @param array $histories
     *
     * @return array
     */
    protected function parseOrderHistory($histories) {
        $ordersHistory = array();
        $ids = array();
        foreach ($histories as $history){
            $ordersHistory[$history['orderID']] = $history;
            $ids[] = $history['orderID'];
        }

        $this->ordersHistory = $ordersHistory;

        return $ids;
    }

    /**
     * Gets date range from turnaround time.
     *
     * @param int $turnAroundTime
     *
     * @return array
     *
     * @throws Exception
     */
    public function getDateRange($turnAroundTime)
    {
        $dateRange['from'] = new DateTime($turnAroundTime);
        $dateRange['to'] = new DateTime('now');

        return $dateRange;
    }

    /**
     * Verifies the shop ID and Secret from eKomi.
     *
     * @param array $configurationData
     *
     * @return mixed
     */
    public function verifyAccount($configurationData)
    {
        $apiUrl = self::URL_GET_SETTINGS;
        $apiUrl .= "?auth={$configurationData['shop-id']}|{$configurationData['shop-secret']}";
        $apiUrl .= '&version=cust-1.0.0&type=request&charset=iso&app=PD-Shopware-5';
        $response = $this->doCurl($apiUrl, 'GET');
        if (self::GET_SETTINGS_RESPONSE == $response) {
            return false;
        }

        return true;
    }

    /**
     * Gets Configuration Data.
     *
     * @param Object $config
     *
     * @return array
     */
    public function getConfigurationData($config)
    {
        $cacheManager = Shopware()->Container()->get('shopware.cache_manager');
        $cacheManager->clearConfigCache();

        $configurationData = array(
            'plugin-enabled' => $config->get('plugin-enabled', self::PLUGIN_STATUS_DISABLE),
            'shop-id' => $config->get('shop-id'),
            'shop-secret' => $config->get('shop-secret'),
            'terms-and-conditions' => $config->get('terms-and-conditions'),
            'product-reviews' => $config->get('product-reviews'),
            'api-mode' => $config->get('api-mode'),
            'product-identifier' => $config->get('product-identifier'),
            'exclude-products' => $config->get('exclude-products'),
            'turnaround-time' => $config->get('turnaround-time'),
            'order-last-updated' => $config->get('order-last-updated'),
            'export-method' => $config->get('export-method'),
            'show-prc-widget' => $config->get('show-prc-widget'),
            'prc-widget-token' => $config->get('prc-widget-token'),
        );
        foreach ($config->get('order-status') as $orderStatus) {
            $configurationData['order-status'][] = $orderStatus;
        }

        return $configurationData;
    }

    /**
     * Gets post data for Plugins dashboard Api.
     *
     * @param array                      $orderData
     * @param array                      $configurationData
     * @param \Shopware\Models\Shop\Shop $shop
     * @param string                     $method
     *
     * @return array
     */
    public function getPostFields($orderData, $configurationData, $shop, $method)
    {
        $orderData = $this->fillCanonicalUrls($orderData, $shop, $method);

        return array(
            'shop_id' => $configurationData['shop-id'],
            'interface_password' => $configurationData['shop-secret'],
            'order_data' => $orderData,
            'mode' => $configurationData['api-mode'],
            'product_identifier' => $configurationData['product-identifier'],
            'exclude_products' => $configurationData['exclude-products'],
            'product_reviews' => (int) $configurationData['product-reviews'],
            'plugin_name' => self::PLUGIN_NAME,
        );
    }

    /**
     * Sends Order data to eKomi Plugins dashboard.
     *
     * @param array                      $orderData
     * @param array                      $configurationData
     * @param \Shopware\Models\Shop\Shop $shop
     * @param string                     $method
     *
     * @return mixed|string
     *
     * @throws Exception
     */
    public function sendOrderData($orderData, $configurationData, $shop = null, $method = 'hook')
    {
        $response = '';
        if (
            !empty($orderData)
            && in_array($orderData['status'], $configurationData['order-status'])
            && $configurationData['terms-and-conditions'] == self::PLUGIN_TERMS_AND_CONDITIONS_AGREED
        ) {
            $boundary = md5(time());
            $httpHeader = array('ContentType:multipart/form-data;boundary='.$boundary);
            $postFields = json_encode($this->getPostFields($orderData, $configurationData, $shop, $method));
            $response = $this->doCurl(self::URL_TO_SEND_DATA, 'PUT', $httpHeader, $postFields);
        }

        return $response;
    }

    /**
     * Gets url redirected to.
     *
     * @param string $url
     *
     * @return mixed
     */
    public function getUrlRedirectedTo($url)
    {
        $newUrl = $this->doCurl($url, 'GET', $httpHeader = array(), '', true, true, true, true);

        return $newUrl;
    }

    /**
     * Makes a curl request.
     *
     * @param string $requestUrl     Api End point url
     * @param string $requestType    Api Request type
     * @param array  $httpHeader     Header
     * @param string $postFields     The post data to send
     * @param bool   $returnCurlInfo Return Curl info
     * @param bool   $header         The header value
     * @param bool   $noBody         If no body is to return
     * @param bool   $followLocation Follow location value
     *
     * @return bool|mixed|string
     */
    public function doCurl(
        $requestUrl,
        $requestType,
        $httpHeader = array(),
        $postFields = '',
        $returnCurlInfo = false,
        $header = false,
        $noBody = false,
        $followLocation = false
    ) {
        try {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $requestUrl);
            curl_setopt($ch, CURLOPT_HEADER, $header);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $requestType);
            curl_setopt($ch, CURLOPT_NOBODY, $noBody);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, $followLocation);
            if (!empty($httpHeader)) {
                curl_setopt($ch, CURLOPT_HTTPHEADER, $httpHeader);
            }

            if (!empty($postFields)) {
                curl_setopt($ch, CURLOPT_POSTFIELDS, $postFields);
            }

            $response = curl_exec($ch);
            if ($returnCurlInfo) {
                $response = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);
            }

            curl_close($ch);

            return $response;
        } catch (Exception $exception) {
            $this->log($exception->getMessage(), 'Exception');

            return $exception->getMessage();
        }
    }

    /**
     * Logs the errors.
     *
     * @param string $mixed
     * @param string $message
     */
    private function log($mixed, $message = '')
    {
        if (Shopware()->Container()->get('config')->get('version') < self::MAX_SHOPWARE_VERSION) {
            $filePath = 'logs/ekomi-shopware-log.log';
            $str = $message.PHP_EOL;
            ob_start();
            var_dump($mixed);
            $str .= ob_get_clean();
            $str .= PHP_EOL;
            file_put_contents($filePath, $str, FILE_APPEND);
        } else {
            Shopware()->Container()->get('pluginlogger')->error($message  . $mixed);
        }
    }
}
